%     Copyright (C) 2017 Imatest LLC
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

function forwardCoeffs = invert_distortion_poly(inverseCoeffs)
% forwardCoeffs = invert_distortion_poly(inverseCoeffs)
% 
% Returns the coefficients of a polynomial which inverts the radial distortion-correcting polynomial 
% mapping returned by Imatest. Thus the output polynomial will apply the distortion that was seen in
% the image. 
% 
% The input polynomial describes a direct mapping from distorted to undistorted radial distances:
%
% r_u = P(r_d)
% 
% where   r_d = radial distance of a point in the distorted (observed) image
%         r_u = radial distance of that point in an undistorted (ideal) image
%         P() is a polynomial function
%
% Note, both arguments are in standard MATLAB polynomial format. See polyval.m help for more.
% Polynomials read from Imatest plots/outputs may need to be translated (reordered and/or augmented 
% with zeros) to fit this form.
% 
% Technically, this function coudld be applied to invert either set of polynomial coefficients into
% the other, and should be (close to) idempotent. 
% 
% - - Input - -
% inverseCoeffs : row vector of polynomial coefficients describing P() above, 3rd or 5th order
%
% - - Output - -
% forwardCoeffs : row vector of polynomial coefficients describing r_d = f_inv(r_u), 5th order


% Inversion is done numerically by generating some undistorted points, calculating some distorted
% version of them, and then fitting a polynomial.

rDistorted = linspace(0,1,10); % radial distortion is <= 5th order, so 10 points is more than enough
rUndistorted = polyval(inverseCoeffs,rDistorted);
forwardCoeffs = polyfit(rUndistorted,rDistorted,5);