%     Copyright (C) 2017 Imatest LLC
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

function forwardCoeffs = lgd2forward_poly(lgdCoeffs)
% forwardCoeffs = lgd2forward_poly(lgdCoeffs)
%
% Returns the coefficients of a polynomial which applies the radial Local Geometric Distortion (LGD)
% described by the input polynomial coefficients as returned by Imatest. Thus the output polynomial 
% will apply the distortion that was seen in the image. 
% 
% The LGD function is defined as:
% 
% LGD(r_d)  =  100*(r_d - r_u)/r_u  =  P(r_d)
%
% where   r_d = radial distance of a point in the distorted (observed) image
%         r_u = radial distance of that point in an undistorted (ideal) image
%         P() is a polynomial function
% 
% Note, both arguments are in standard MATLAB polynomial format. See polyval.m help for more.
% Polynomials read from Imatest plots/outputs may need to be translated (reordered and/or augmented 
% with zeros) to fit this form.
%
% - - Input - - 
% lgdCoeffs : row vector of polynomial coefficients describing the LGD() above, typically 3rd order
%
% - - Output - -
% forwardCoeffs : row vector of polynomial coefficients describing r_u = f_inv(r_d), 5th order

% Rearrange the above LDG formula to get f_u = f(r_d), then convert from this rational polynomial
% form to a normal polynomial by numerical fitting.

rDistorted = linspace(0,1,100);
rUndistorted = rDistorted./(polyval(lgdCoeffs, rDistorted)/100 + 1);
forwardCoeffs = polyfit(rUndistorted, rDistorted,5);